<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

class WV8_Settings_Synchronizer extends WV_Service_Property {
	protected $config;
	protected $db;
	protected $langService;
	protected $languages;

	public function __construct(sly_Configuration $config, sly_DB_PDO_Persistence $db, sly_Service_Language $langService = null) {
		$this->config      = $config;
		$this->db          = $db;
		$this->langService = $langService;
		$this->languages   = null;
	}

	public function synchronize(array $oldData, array $newData) {
		$this->db->transactional(array($this, 'loadFromArray'), array($newData, $oldData));
	}

	protected function createProperty($name, array $property) {
		return new WV8_Setting($name, $property);
	}

	protected function onNew(WV_Property $property) {
		$pagename = $property->getPagename();

		if (empty($pagename)) {
			throw new WV8_Exception('Der Seitenname von Setting '.$property->getNamespace().'/'.$property->getName().' darf nicht leer sein.');
		}

		$this->addToDatabase($property);
	}

	protected function onDeleted(WV_Property $property) {
		$this->removeFromDatabase($property);
	}

	protected function onChangedDatatype(WV_Property $oldVersion, WV_Property $newVersion) {
		$this->db->update($oldVersion->getTableName(), array('`values`' => ''), array(
			'namespace' => $oldVersion->getNamespace(),
			'name'      => $oldVersion->getName()
		));
	}

	protected function onChangedParameters(WV_Property $oldVersion, WV_Property $newVersion) {
		$this->onChangedDatatype($oldVersion, $newVersion);
	}

	protected function onChanged(WV_Property $oldVersion, WV_Property $newVersion) {
		if ($oldVersion->isLocal() !== $newVersion->isLocal()) {
			$this->removeFromDatabase($oldVersion);
			$this->addToDatabase($newVersion);
		}
	}

	public function addToDatabase(WV8_Setting $setting) {
		$langs  = $this->getLanguages();
		$values = WV8_Util::encodeValues(WV8_Util::fixValues($setting->getDefault(), $langs));
		$table  = $setting->getTableName();
		$where  = array(
			'namespace' => $setting->getNamespace(),
			'name'      => $setting->getName()
		);

		$count = $this->db->magicFetch($table, 'COUNT(*)', $where);

		if ($count < 1) {
			$where['`values`'] = $values;
			$this->db->insert($table, $where);
		}
	}

	public function removeFromDatabase(WV8_Setting $setting) {
		$this->db->delete($setting->getTableName(), array(
			'namespace' => $setting->getNamespace(),
			'name'      => $setting->getName()
		));
	}

	protected function getLanguages() {
		if ($this->langService !== null && $this->languages === null) {
			$this->languages = $this->langService->findAll(true);
		}

		return $this->languages;
	}
}
