<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\metainfo\Listener;

use sly_Core;
use sly_Form;
use sly_Model_Article;
use sly_Util_Language;
use sly_Service_Article;
use wv\metainfo\Authorisation;
use wv\metainfo\Metainfo;

class Article extends Backend {
	/**
	 * Handler for SLY_ART_COPIED, SLY_ARTICLE_TOUCHED
	 *
	 * Copies the metadata of an article
	 *
	 * @param sly_Model_Article $target  the new article
	 * @param array             $params  additional params
	 */
	public function onArticleCloned(sly_Model_Article $target, array $params) {
		$source = $params['source'];

		// Metadaten kopieren
		$container = Metainfo::getContainer();
		$service   = $container['metainfo-service-value'];

		$service->copy(
			array('object_id' => $source->getId(), 'clang' => $source->getClang(), 'revision' => $source->getRevision()),
			array('object_id' => $target->getId(), 'clang' => $target->getClang(), 'revision' => $target->getRevision()),
			Metainfo::TYPE_ARTICLE
		);

		// Listen-Cache leeren
		Metainfo::clearDataCache();
	}

	/**
	 * Handler for SLY_ART_TO_STARTPAGE
	 *
	 * Copies the metadata of an article
	 *
	 * @param int   $newID   the new category id
	 * @param array $params  additional params
	 */
	public function onArticleToStartarticle($newID, array $params) {
		// add metainfos for the new category
		$newID        = (int) $newID;
		$oldID        = (int) $params['old_cat'];
		$container    = Metainfo::getContainer();
		$valueService = $container['metainfo-service-value'];
		$clangs       = sly_Util_Language::findAll(true);

		foreach ($clangs as $clang) {
			$valueService->copy(
				array('object_id' => $oldID, 'clang' => $clang),
				array('object_id' => $newID, 'clang' => $clang),
				Metainfo::TYPE_CATEGORY
			);
		}

		// remove category metadata for old startarticle
		$valueService->deleteObject($params['old_cat'], Metainfo::TYPE_CATEGORY);

		// clear cache
		Metainfo::clearDataCache();
	}

	/**
	 * Handler for ART_META_FORM
	 *
	 * Creates the metainfo form
	 *
	 * @param  sly_Form $form    the form
	 * @param  array    $params  additional parameters
	 * @return sly_Form          the form
	 */
	public function onArticleMetaForm(sly_Form $form, array $params) {
		if (!Authorisation::canView()) {
			return $form;
		}

		$container    = Metainfo::getContainer();
		$article      = $params['article'];
		$articleType  = $article->getType();
		$isMeta       = $this->isContentmetaPage();
		$propService  = $container['metainfo-service-property'];
		$valueService = $container['metainfo-service-value'];

		$assignedData   = $valueService->findArticleData($article);
		$requiredInfos  = $propService->findArticleProperties($articleType, $isMeta);
		$availableInfos = $isMeta ? $requiredInfos : $propService->findArticleProperties(null, $isMeta);
		$dataToDisplay  = $this->getInfosToDisplay($availableInfos, $assignedData, $requiredInfos);
		$canEdit        = Authorisation::canEdit();

		$form->addClass('wv2_frontend_'.($isMeta ? 'meta' : 'slice'));

		// We need to remember on what article revision we are working, because
		// Sally will only if the type changed create a new revision. By transmitting
		// the original versio, we can properly compare and create a new revision
		// ourselves if needed.
		$form->addHiddenValue('wv2_revision', $article->getRevision());

		if (!$isMeta) {
			// add javascript foo
			$metainfos = array();

			foreach ($availableInfos as $metainfo) {
				foreach ($metainfo->getArticleTypes() as $artType) {

					if (!array_key_exists($artType, $metainfos)) {
						$metainfos[$artType] = array();
					}

					$metainfos[$artType][] = $metainfo->getID();
				}
			}

			sly_Core::getLayout()->addJavaScript('var wv2={}; wv2.infosPerType='.json_encode($metainfos).';');
		}

		include _WV2_PATH.'templates/articles/metainfos.phtml';

		return $form;
	}

	/**
	 * Handler for ART_META_FORM_FIELDSET
	 *
	 * Creates a formular for additinal actions
	 *
	 * @param  sly_Form $form    the form
	 * @param  array    $params  additional parameters
	 * @return sly_Form          the form
	 */
	public function onArticleMetaFieldset(sly_Form $form, array $params) {
		if (!Authorisation::canEdit() || !sly_Util_Language::isMultilingual()) {
			return $form;
		}

		$container = Metainfo::getContainer();
		$request   = $container->getRequest();

		if ($request->post('wv2_copymetadata', 'boolean')) {
			$article_id  = $request->post('article_id', 'int');
			$clang       = $request->post('clang', 'int');
			$revision    = $request->post('revision', 'int');
			$targetLangs = array_unique($request->postArray('wv2_copy_to', 'int'));

			if (empty($targetLangs)) {
				$container->getFlashMessage()->addWarning(t('no_language_selected'));
				return;
			}

			$articleService = $container->getArticleService();
			$valueService   = $container['metainfo-service-value'];
			$source         = $articleService->findByPK($article_id, $clang, $revision);

			foreach ($targetLangs as $targetLang) {
				if ($clang === $targetLang) {
					$container->getFlashMessage()->addWarning(t('metainfo_languages_equal'));
					continue;
				}

				$target = $articleService->findByPK($article_id, $targetLang, sly_Service_Article::FIND_REVISION_LATEST);

				if ($source->getType() !== $target->getType()) {
					$container->getFlashMessage()->addWarning(t('metainfo_types_not_equal'));
					continue;
				}

				$valueService->copy(
					array('object_id' => $source->getId(), 'clang' => $source->getClang(), 'revision' => $source->getRevision()),
					array('object_id' => $target->getId(), 'clang' => $target->getClang(), 'revision' => $target->getRevision()),
					Metainfo::TYPE_ARTICLE
				);

				$container->getFlashMessage()->addInfo(t('metainfo_metadata_copied'));
			}

			Metainfo::clearDataCache();

			$response = $container->getApplication()->redirectResponse('contentmeta', 'index', $params = array('article_id' => $article_id, 'clang' => $clang, 'revision' => $revision));
			$response->send();
		}

		$clang  = (int) $params['clang'];
		$submit = new \sly_Form_Input_Button('submit', 'wv2_copymetadata', t('metainfo_copymetadata'));

		$to = \sly_Backend_Form_Helper::getLanguageSelect('wv2_copy_to');
		$to->setLabel(t('to_language'));
		$to->setMultiple(true);

		$form->beginFieldset(t('metainfo_copy_between_languages'), '', 2);
		$form->add($to);
		$form->add(new \sly_Form_ButtonBar(compact('submit')));

		return $form;
	}

	/**
	 * Handler for ART_META_UPDATED
	 *
	 * @param sly_Model_Article $article  the article that was edited
	 */
	public function onArticleMetaUpdated(sly_Model_Article $article) {
		if (!Authorisation::canEdit()) {
			return;
		}

		// check if we need to create a new version or if someone else has already
		// taken care of that for us
		// TODO: Move this logic into the serializer, so it can check for any real
		// changes before an actual copy is created.

		$container   = Metainfo::getContainer();
		$oldRevision = $container['sly-request']->post('wv2_revision', 'int', 0);
		$newRevision = $article->getRevision();

		if ($oldRevision == $newRevision) {
			$artService = $container['sly-service-article'];
			$article    = $artService->touch($article);
		}

		// serialize and store the form data

		$serializer = $container['metainfo-serializer'];
		$isMeta     = $this->isContentmetaPage();
		$errors     = $serializer->processArticleForm($article, $isMeta);

		if (is_array($errors)) {
			self::showErrors($errors, Metainfo::TYPE_ARTICLE);
		}
	}

	/**
	 * Handler for SLY_ART_TYPE
	 *
	 * Updates all associated values
	 *
	 * @param sly_Model_Article $article  the article
	 * @param array             $params   additional parameters
	 */
	public function onArticleTypeUpdated(sly_Model_Article $article, array $params) {
		$oldType = $params['old_type'];
		$newType = $article->getType();

		if ($oldType === $newType) {
			return;
		}

		$container    = Metainfo::getContainer();
		$valueService = $container['metainfo-service-value'];
		$propService  = $container['metainfo-service-property'];
		$infosOldType = array_keys($propService->findArticleProperties($oldType));
		$infosNewType = array_keys($propService->findArticleProperties($newType));

		///////////////////////////////////////////////////////////////////////
		// Lösche alle Metadaten von Metainfos, die der neue Typ nicht besitzt

		$infosToDelete = array_diff($infosOldType, $infosNewType);

		if (!empty($infosToDelete)) {
			foreach ($infosToDelete as $idx => $metainfo) {
				$valueService->delete($article->getId(), $article->getClang(), $article->getRevision(), Metainfo::TYPE_ARTICLE, $metainfo);
			}
		}

		///////////////////////////////////////////////////////////////////////
		// add the new metainfo's default value to the article

		$infosToAdd  = array_diff($infosNewType, $infosOldType);
		$newMetadata = array();

		foreach ($infosToAdd as $metainfo) {
			$info                   = $propService->findProperty(Metainfo::TYPE_ARTICLE, $metainfo);
			$newMetadata[$metainfo] = $info->serialize($info->getDefault());
		}

		$valueService->insertArticleData($article, $newMetadata);

		Metainfo::clearDataCache();
	}

	/**
	 * Handler for SLY_ART_ADDED
	 *
	 * @param sly_Model_Article  the added article
	 */
	public function onArticleAdded(sly_Model_Article $article) {
		$type = $article->getType();

		// do nothing if no articletype is set

		if (empty($type)) {
			return;
		}

		// Standardwerte übernehmen
		$container    = Metainfo::getContainer();
		$valueService = $container['metainfo-service-value'];
		$propService  = $container['metainfo-service-property'];
		$metainfos    = $propService->findArticleProperties($type);

		foreach ($metainfos as $name => $metainfo) {
			$metainfos[$name] = $metainfo->getDefault();
		}

		$valueService->insertArticleData($article, $metainfos);
	}

	/**
	 * Handler for SLY_ART_DELETED
	 *
	 * Only clears the cache because old revisions are stored forever
	 */
	public function onArticleDeleted() {
		Metainfo::clearDataCache();
	}

	public function onArticleRevisionDeleted(sly_Model_Article $article) {
		$container    = Metainfo::getContainer();
		$valueService = $container['metainfo-service-value'];

		$valueService->delete($article->getId(), $article->getClang(), $article->getRevision(), Metainfo::TYPE_ARTICLE);

		Metainfo::clearDataCache();
	}

	/**
	 * Handler for SLY_ART_UPDATED
	 */
	public function onArticleUpdated() {
		Metainfo::clearDataCache();
	}

	protected function isContentmetaPage() {
		return sly_Core::getCurrentControllerName() === 'contentmeta';
	}
}
