<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\metainfo;

use sly\Assets\Util;
use wv\BabelCache\Util as BabelCacheUtil;
use wv\metainfo\Property\Property as MetainfoProperty;

class Metainfo {
	const CACHE_NAMESPACE = 'metainfo';
	const TYPE_ARTICLE    = 0; ///< int  Metainfo-Typ für Artikel
	const TYPE_CATEGORY   = 1; ///< int  Metainfo-Typ für Kategorien
	const TYPE_MEDIUM     = 2; ///< int  Metainfo-Typ für Medien
	const TYPE_USER       = 3; ///< int  Metainfo-Typ für Benutzer

	protected static $container;

	public static function setContainer(\sly_Container $container) {
		self::$container = $container;
	}

	public static function initLayout() {
		$layout   = self::getContainer()->getLayout();
		$me       = 'webvariants/metainfo';
		$devutils = 'webvariants/developer-utils';

		$layout->addCSSFile(Util::addOnUri($me, 'css/metainfo.less'));
		$layout->addJavaScriptFile(Util::addOnUri($devutils, 'js/jquery.cookie.min.js'));
		$layout->addJavaScriptFile(Util::addOnUri($devutils, 'js/jquery.wvtogglefieldset.min.js'));
		$layout->addJavaScriptFile(Util::addOnUri($me, 'js/metainfo.min.js'));
	}

	/**
	 * @return \sly_Container
	 */
	public static function getContainer() {
		return self::$container;
	}

	/**
	 *
	 * @throws \wv\metainfo\Exception  if the object has no meta type
	 *
	 * @param  object $obj  the object
	 * @return string       type of the object
	 */
	public static function getTypeForObject($obj) {
		if ($obj instanceof \sly_Model_Article) {
			return self::TYPE_ARTICLE;
		}
		elseif ($obj instanceof \sly_Model_Category) {
			return self::TYPE_CATEGORY;
		}
		elseif ($obj instanceof \sly_Model_Medium) {
			return self::TYPE_MEDIUM;
		}
		elseif ($obj instanceof \sly_Model_User) {
			return self::TYPE_USER;
		}

		throw new Exception(t('metainfo_unkown_object'));
	}

	public static function getSupportedTypes() {
		return array(self::TYPE_ARTICLE, self::TYPE_CATEGORY, self::TYPE_MEDIUM, self::TYPE_USER);
	}

	/**
	 *
	 * @param  string $type
	 * @throws \wv\metainfo\Exception
	 */
	public static function validateType($type) {
		if (!in_array($type, self::getSupportedTypes())) {
			throw new Exception(t('metainfo_unkown_object_type', $type));
		}
	}

	public static function clearDataCache() {
		self::getCache()->clear(self::CACHE_NAMESPACE, true);
	}

	/**
	 *
	 * @param  string $articleType  the artikeltype
	 * @param  bool   $onlineOnly   find only online articles
	 * @param  string $sortby       sort column
	 * @param  string $direction    sort direction
	 * @param  int    $limit        Limit
	 * @param  int    $offset       Offset
	 * @param  int    $clang        clang id or null for all
	 * @return array                list of \sly_Model_Article
	 */
	public static function getArticlesByType($articleType, $onlineOnly = true, $sortby = 'updatedate', $direction = 'ASC', $limit = null, $offset = null, $clang = null) {
		$clang   = WV_Sally::clang($clang);
		$service = self::$container->getArticleService();
		$where   = array('type' => $articleType, 'clang' => $clang);

		// cut off table name (might be given when the metalist helper tries
		// to create a list and is given 'article.pos' as a sorting order.
		// In getObjectsWithMetaData() we allow this, so we should allow this here, too.
		if (strpos($sortby, '.') !== false) {
			$sortby = substr($sortby, strpos($sortby, '.') + 1);
		}

		return $service->find($where, null, $sortby.' '.$direction, $offset, $limit, null, $onlineOnly ? \sly_Service_Article::FIND_REVISION_ONLINE : \sly_Service_Article::FIND_REVISION_LATEST);
	}

	/**
	 *
	 * @param  mixed   $articleType  der Artikeltyp als ID oder Name
	 * @param  boolean $onlineOnly
	 * @param  int     $clang
	 * @return int                   die Anzahl der zugehörigen Artikel
	 */
	public static function getArticleCountByType($articleType, $onlineOnly = true, $clang = null) {
		$namespace = self::CACHE_NAMESPACE.'.acount';
		$cache     = self::getCache();
		$key       = BabelCacheUtil::generateKey('by_type', $articleType, $onlineOnly, $clang);
		$count     = $cache->get($namespace, $key, null);

		if ($count === null) {
			$count = count(self::getArticlesByType($articleType, $onlineOnly, null, null, null, null, $clang));
			$cache->set($namespace, $key, $count);
		}

		return $count;
	}

	/**
	 * Gets the available properties, not the data!
	 *
	 * @return array  a list of \wv\metainfo\Property
	 */
	public static function getAllObjectMetaInfos() {
		return self::$container['metainfo-service-property']->findAll();
	}

	/**
	 * Gets the available properties, not the data!
	 *
	 * @param  string $articleType  filter by articletype
	 * @param  string $metapage     filter by metapage attribute
	 * @return array                a list of \wv\metainfo\Property
	 */
	public static function getAllArticleMetaInfos($articleType = null, $metapage = null) {
		return self::$container['metainfo-service-property']->findArticleProperties($articleType, $metapage);
	}

	/**
	 * Gets the available properties, not the data!
	 *
	 * @return array  a list of \wv\metainfo\Property
	 */
	public static function getAllCategoryMetaInfos() {
		return self::$container['metainfo-service-property']->findCategoryProperties();
	}

	/**
	 * Gets the available properties, not the data!
	 *
	 * @return array  a list of \wv\metainfo\Property
	 */
	public static function getAllMediumMetaInfos() {
		return self::$container['metainfo-service-property']->findMediumProperties();
	}

	/**
	 * Gets the available properties, not the data!
	 *
	 * @return array  a list of \wv\metainfo\Property
	 */
	public static function getAllUserMetaInfos() {
		return self::$container['metainfo-service-property']->findUserProperties();
	}

	/**
	 *
	 * return an assoc array of [name: value]
	 *
	 * @param  int     $object    the ID if the object
	 * @param  mixed   $metainfo  name of the metainfo
	 * @param  mixed   $default   the default value if the info is not available
	 * @param  int     $clang     selected language
	 * @param  int     $type      the object type (Metainfo::TYPE-Contant)
	 * @param  boolean $raw       if true, return serialized data
	 * @return array              array of metainfo values
	 */
	public static function objectData($objectId, $metainfo, $default, $clang, $revision, $type, $raw) {
		self::validateType($type);

		// prepare data
		$container    = self::getContainer();
		$cache        = self::getCache();
		$clangs       = ($clang === \WV_Sally::CLANG_ALL) ? \sly_Util_Language::findAll(true) : array(\WV_Sally::clang($clang));
		$objectId     = (int) $objectId;
		$revision     = (int) $revision;
		$namespace    = self::CACHE_NAMESPACE.'.data';

		// force revision 0 for categories
		if ($type === self::TYPE_CATEGORY) {
			$revision = 0;
		}

		$base_key     = 'ocrt_' . $objectId . '_' . $clang . '_' . $revision . '_' . $type . '_';
		$rawKey       = $base_key . 'r';
		$deserialKey  = $base_key . 'd';
		$rawData      = null;
		$deserialized = null;

		// check cache
		if (!$raw) {
			$deserialized = $cache->get($namespace, $deserialKey, null);
		}

		if ($deserialized === null) {
			$rawData = $cache->get($namespace, $rawKey, null);
		}

		// query database
		if ($deserialized === null && $rawData === null) {
			$dataservice = $container['metainfo-service-value'];
			$rawData = array();
			foreach ($clangs as $clangId) {
				$data = $dataservice->findObjectData($objectId, $clangId, $revision, $type);

				foreach ($data as $name => $value) {
					$rawData[$name][$clangId] = $value;
				}
			}

			// store raw data
			$cache->set($namespace, $rawKey, $rawData);
		}

		// deserialize raw values
		if (!$raw && $deserialized === null) {
			$propservice  = $container['metainfo-service-property'];
			$deserialized = array();

			foreach ($rawData as $name => $date) {
				$info   = $propservice->findProperty($type, $name);
				$params = $info->getParams();

				foreach ($clangs as $clangId) {
					$value = $info->deserialize($date[$clangId]);

					if ($info->isValueEmpty($value)) {
						$value = null;
					}

					$deserialized[$name][$clangId] = $value;
				}
			}

			// store deserialized data
			$cache->set($namespace, $deserialKey, $deserialized);
		}

		$return = $raw ? $rawData : $deserialized;

		if (($clang !== \WV_Sally::CLANG_ALL)) {
			foreach ($return as $name => $date) {
				$return[$name] = $date[$clang];
			}
		}

		if ($metainfo !== null) {
			if (!array_key_exists($metainfo, $return)) {
				throw new Exception('Could not find meta data for "'.$metainfo.'" on object '.$objectId.'/'.$clang.'!');
			}

			return array($metainfo => $return[$metainfo]);
		}

		return $return;
	}

	public static function articleData(\sly_Model_Article $article, $metainfo = null, $clang = \WV_Sally::CLANG_CURRENT, $raw = false) {
		return self::objectData($article->getId(), $metainfo, $default, $article->getClang(), $article->getRevision(), Metainfo::TYPE_ARTICLE, $raw);
	}

	public static function categoryData(\sly_Model_Category $category, $metainfo = null, $clang = \WV_Sally::CLANG_CURRENT, $raw = false) {
		return self::objectData($category->getId(), $metainfo, $default, $category->getClang(), $category->getRevision(), Metainfo::TYPE_CATEGORY, $raw);
	}

	public static function mediumData(\sly_Model_Medium $medium, $metainfo = null, $default = null, $clang = \WV_Sally::CLANG_CURRENT, $raw = false) {
		return self::objectData($medium->getId(), $metainfo, $default, $clang, $medium->getRevision(), Metainfo::TYPE_MEDIUM, $raw);
	}

	public static function userData(\sly_Model_User $user, $metainfo = null, $default = null, $clang = \WV_Sally::CLANG_CURRENT, $raw = false) {
		return self::objectData($user->getId(), $metainfo, $default, $clang, $user->getRevision(), Metainfo::TYPE_USER, $raw);
	}

	/**
	 * Find objects with associated metadata
	 *
	 * Return an array of \sly_Model_Article / \sly_Model_Category /
	 * \sly_Model_Medium / \sly_Model_User which contains a metainfo. Additionally
	 * the value of the metainfo can be specified.
	 *
	 * It is recommened to use the getXXXWithMetaData methods.
	 *
	 * The sort query must begiven in the "table.column" format.
	 * "table" is a fake format which maps real the sally or metainfo tables
	 *
	 *	- article
	 *	- category
	 *	- user
	 *	- medium OR file
	 *	- meta OR metadata
	 *
	 * If an invalid table is given an exception is thrown
	 *
	 * @throws \UnexpectedValueException
	 * @param  mixed   $metainfo     name of the metainfo
	 * @param  boolean $onlineOnly   wenn true, werden nur online Artikel berücksichtigt (default: true)
	 * @param  string  $value        the value to match, or null for all
	 * @param  int     $operator     Anweisungen an den Datentyp, wie die Suche nach dem Wert ($value) zu erfolgen hat
	 * @param  string  $sort         an optinal "ORDER BY"-Clause (without "ORDER BY")
	 * @param  mixed   $articleType  name of the articletype of null for all
	 * @param  int     $clang        language (WV_Sally::CLANG-Constants)
	 * @param  int     $type         the Type (Metainfo::TYPE-constants)
	 * @return array                 list of matching Objects
	 */
	protected static function getObjectsWithMetaData($objectType, $metainfo, $onlineOnly = true, $value = null, $operator = null, $sort = null, $articleType = null, $clang = null) {
		$container = self::getContainer();
		$clang     = \WV_Sally::clang($clang);
		$cache     = self::getCache();
		$namespace = self::CACHE_NAMESPACE.'.objects';
		$cacheKey  = BabelCacheUtil::generateKey('metainfo', $objectType, $metainfo, $onlineOnly, $value, $operator, $sort, $articleType, $clang);
		$data      = $cache->get($namespace, $cacheKey, false);
		$return    = array();

		if ($data === false) {
			list($sortTable, $sortColumn) = strpos($sort, '.') === false ? array('', $sort) : explode('.', $sort, 2);

			switch ($sortTable) {
				case 'meta':
				case 'metadata':
					$sortTable = 'm';

				case 'article':
				case 'category':
				case 'medium':
				case 'file':
				case 'user':
					$sortTable = 'o';
					break;

				case '':
					break;

				default:
					throw new \UnexpectedValueException('Unexpected sort table "'.$sortTable.'" given.');
			}

			$sort        = $sortTable === '' ? $sortColumn : $sortTable.'.'.$sortColumn;
			$persistence = self::getContainer()->getPersistence();
			$where       = array();
			$join        = self::getObjectTableJoins($objectType, $persistence->getPrefix());

			switch ($objectType) {
				case self::TYPE_ARTICLE:
					if ($articleType) {
						$where['o.type'] = $articleType;
					}
					break;

				case self::TYPE_MEDIUM:
				case self::TYPE_USER:
					if ($onlineOnly === true) {
						$where['o.status'] = 1;
					}
					break;
			}

			$where['m.metainfo'] = $metainfo;
			$where['m.clang']    = $clang;

			// query the database
			$select = 'm.object_id, m.revision, m.value';

			$persistence->select('wv2_meta m', $select, $where, null, $sort, null, null, null, $join);
			$rows = $persistence->all();

			// empty result set?
			if (empty($rows)) {
				$cache->set($namespace, $cacheKey, array());
				return array();
			}

			// get datatype to decode the value
			if ($value !== null) {
				$property = $container['metainfo-service-property']->findProperty($objectType, $metainfo);
			}

			$data = array();

			// check values
			foreach ($rows as $row) {
				if ($value === null || $property->dataTypeCall('isValueContained', array($value, $row['value'], $property->getParams(), $operator))) {
					$data[] = array('object_id' => (int) $row['object_id'], 'revision' => $row['revision']);
				}
			}

			$cache->set($namespace, $cacheKey, $data);
		}

		foreach ($data as $date) {
			$obj = self::getObject($objectType, $date['object_id'], $clang, $date['revision']);

			if ($obj) {
				if (($objectType === self::TYPE_ARTICLE || $objectType === self::TYPE_ARTICLE) && $onlineOnly && !$obj->isOnline()) {
					continue;
				}

				$return[] = $obj;
			}
		}

		return $return;
	}

	public static function getArticlesWithMetaData($metainfo, $onlineOnly = true, $value = null, $operator = null, $sort = null, $articleType = null, $clang = null) {
		return self::getObjectsWithMetaData(Metainfo::TYPE_ARTICLE, $metainfo, $onlineOnly, $value, $operator, $sort, $articleType, $clang);
	}

	public static function getCategoriesWithMetaData($metainfo, $onlineOnly = true, $value = null, $operator = null, $sort = null, $clang = null) {
		return self::getObjectsWithMetaData(Metainfo::TYPE_CATEGORY, $metainfo, $onlineOnly, $value, $operator, $sort, null, $clang);
	}

	public static function getMediaWithMetaData($metainfo, $value = null, $operator = null, $sort = null) {
		return self::getObjectsWithMetaData(Metainfo::TYPE_MEDIUM, $metainfo, true, $value, $operator, $sort, null, null);
	}

	public static function getUsersWithMetaData($metainfo, $onlineOnly = true, $value = null, $operator = null, $sort = 'user.id') {
		return self::getObjectsWithMetaData(Metainfo::TYPE_USER, $metainfo, $onlineOnly, $value, $operator, $sort, null, null);
	}

	/**
	 * get values for a metainfo
	 *
	 * @param  mixed $metainfo         die Metainfo (kann ein Metainfo-Objekt sein, dann wird $type ignoriert)
	 * @param  bool  $getOnlyExisting  wenn true, werden nur die Werte zurückgegeben, die eine Metainfo auch wirklich angenommen hat
	 * @param  bool  $onlineOnly       wenn true werden nur online Artikel berücksichtigt (nur für Artikel/Kategorien und nur wenn onlyExisting = true)
	 * @param  mixed $objects          ein Array von Objekten, um die Auswahl weiter einzuschränken, oder null
	 * @param  int   $type             der Typ (Metainfo::TYPE-Konstanten)
	 * @return array                   eine Liste von Alternativen
	 */
	public static function getObjectMetaInfoValueSet($type, $metainfo, $getOnlyExisting, $clang, $onlineOnly, $objects) {
		if (is_array($objects)) {
			foreach ($objects as $idx => $object) {
				$objects[$idx] = self::getIdForObject($object, false);
			}
		}

		$container = self::getContainer();
		$clang     = \WV_Sally::clang($clang);

		if ($metainfo instanceof MetainfoProperty) {
			$property = $metainfo;
			$metainfo = $property->getName();
			$type     = $property->getType();
		}
		else {
			$property = $container['metainfo-service-property']->findProperty($type, $metainfo);
		}

		$isAssoc    = $property->datatypeCall('usesAssociativeResults');
		$cache      = $container->getCache();
		$namespace  = self::CACHE_NAMESPACE.'.valuesets';
		$cacheKey   = BabelCacheUtil::generateKey('metainfo', $type, $metainfo, $getOnlyExisting, $onlineOnly, $objects, $clang);
		$cachedData = $cache->get($namespace, $cacheKey);

		if ($cachedData !== null) {
			return $cachedData;
		}

		$datalist = array();

		if ($getOnlyExisting) {
			if ($onlineOnly && ($type === self::TYPE_ARTICLE || $type === self::TYPE_CATEGORY)) {
				$where['clang'] = $clang;

				if ($type === self::TYPE_ARTICLE) {
					$where['type'] = $property->getArticleTypes();
					$service       = $container->getArticleService();
				}
				else {
					$service	= $container->getCategoryService();
				}

				if (is_array($objects)) {
					$where['id'] = $objects;
				}

				$objects = $service->find($where, null, null, null, null, null, \sly_Service_Article::FIND_REVISION_ONLINE);

				foreach ($objects as $idx => $object) {
					$objects[$idx] = array($object->getId(), $object->getRevision());
				}

				// stop here if there are no online articles
				if (empty($objects)) {
					return array();
				}
			}
			elseif ($onlineOnly) {
				trigger_error('You set $onlineOnly to true, but the metatype you are using ('.$type.') is not supported. You have to provide a pre-filtered list of online objects to work around this.', E_USER_WARNING);
			}

			$db     = $container->getPersistence();
			$select = 'DISTINCT value';
			$table  = 'wv2_meta';
			$where  = array(
				'meta_type' => $type,
				'metainfo'  => $metainfo,
				'clang'     => $clang
			);

			if (is_array($objects)) {
				foreach ($objects as $object) {
					$where['object_id'] = $object[0];
					$where['revision']  = $object[1];

					$rows[] = $db->fetch($table, $select, $where);
				}
			}
			else {
				$db->select($table, $select, $where);
				$rows = $db->all();
			}

			foreach ($rows as $row) {
				$value    = $property->deserialize($row['value']);
				$datalist = $isAssoc ? sly_merge($datalist, $value) : array_merge($datalist, sly_makeArray($value));
			}

			$datalist = array_unique($datalist);
		}
		else {
			$datalist = $property->datatypeCall('extractValuesFromParams',array($property->getParams()));
		}

		$cache->set($namespace, $cacheKey, $datalist);
		return $datalist;
	}

	public static function getArticleMetaInfoValueSet($metainfo, $getOnlyExisting = false, $onlineOnly = true, $objects = null, $clang = null) {
		return self::getObjectMetaInfoValueSet(self::TYPE_ARTICLE, $metainfo, $getOnlyExisting, $clang, $onlineOnly, $objects);
	}

	public static function getCategoryMetaInfoValueSet($metainfo, $getOnlyExisting = false, $onlineOnly = true, $objects = null, $clang = null) {
		return self::getObjectMetaInfoValueSet(self::TYPE_CATEGORY, $metainfo, $getOnlyExisting, $clang, $onlineOnly, $objects);
	}

	public static function getMediumMetaInfoValueSet($metainfo, $getOnlyExisting = false, $objects = null, $clang = null) {
		return self::getObjectMetaInfoValueSet(self::TYPE_MEDIUM, $metainfo, $getOnlyExisting, $clang, false, $objects);
	}

	public static function getUserMetaInfoValueSet($metainfo, $getOnlyExisting = false, $objects = null, $clang = null) {
		return self::getObjectMetaInfoValueSet(self::TYPE_USER, $metainfo, $getOnlyExisting, $clang, false, $objects);
	}

	/**
	 *
	 * @param  int $type      one of Metainfo::TYPE_-Constants
	 * @param  int $objectId  object id
	 * @param  int $clang     object clang (used if available for object)
	 * @param  int $revision  object revision (used if available for object)
	 * @return sly_Model_XXX  instance or null
	 */
	public static function getObject($type, $objectId, $clang, $revision) {
		switch ($type) {
			case self::TYPE_ARTICLE:
				return \sly_Util_Article::findById($objectId, $clang, $revision);

			case self::TYPE_CATEGORY:
				return \sly_Util_Category::findById($objectId, $clang);

			case self::TYPE_MEDIUM:
				return \sly_Util_Medium::findById($objectId);

			case self::TYPE_USER:
				return \sly_Util_User::findById($objectId);

			default:
				return null;
		}
	}

	/**
	 * finds the object id
	 *
	 * @param  mixed $object     int, null, sly_Model_Article, sly_Model_Category, sly_Model_Medium, sly_Model_User
	 * @param  bool  $allowNull  wenn true, wird für $object = null auch null zurückgegeben
	 * @return int               die ID oder -1 falls keine gefunden wurde
	 */
	private static function getIdForObject($object, $allowNull = true) {
		if ($object === null && $allowNull) {
			return null; /* <- null means all in the metainfo universe */
		}
		elseif (\sly_Util_String::isInteger($object)) {
			return (int) $object; /* <- article, category, medium, user? whatever. */
		}
		elseif (is_object($object)) {
			self::validateType(self::getTypeForObject($object));
			return $object->getId();
		}
		else {
			trigger_error(t('metainfo_cant_identify_object', $object, gettype($object)), E_USER_WARNING);
		}

		return -1;
	}

	private static function getObjectTableJoins($objectType, $prefix) {
		switch ($objectType) {
			case self::TYPE_ARTICLE:
			case self::TYPE_CATEGORY:
				$objectTable = $prefix.'article';
				$join = 'LEFT JOIN ~ o ON o.id = m.object_id AND o.clang = m.clang';
				break;

			case self::TYPE_MEDIUM:
				$objectTable = $prefix.'file';
				$join = 'LEFT JOIN ~ o ON o.id = m.object_id';
				break;

			case self::TYPE_USER:
				$objectTable = $prefix.'user';
				$join = 'LEFT JOIN ~ o ON o.id = m.object_id';
				break;
		}

		$join .= ' AND o.revision = m.revision';
		$join  = str_replace('~', $objectTable, $join);

		return $join;
	}

	/**
	 * @return \wv\BabelCache\CacheInterface
	 */
	private static function getCache() {
		return self::$container->getCache();
	}
}
