<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\realurl2;

use sly_DB_PDO_Persistence;
use sly_Model_Article;
use sly_Util_Article;
use wv\BabelCache\CacheInterface;

/**
 * @since  v4.0
 * @author Christoph
 */
class Resolver {
	protected $storage;
	protected $cache;
	protected $db;
	protected $cacheHit;

	public function __construct(Storage $storage, CacheInterface $cache, sly_DB_PDO_Persistence $db) {
		$this->storage  = $storage;
		$this->cache    = $cache;
		$this->db       = $db;
		$this->cacheHit = false;
	}

	/**
	 * @return _WV24_URL
	 */
	public function resolveUrl($url) {
		$url = Util::cleanUrl($url);

		// try the raw URL itself
		$urlInstance = $this->findUrl($url);

		if ($urlInstance !== null) {
			if (!$this->cacheHit) {
				$this->writeCache($url, $urlInstance);
			}

			return $urlInstance;
		}

		// try '/index.html' and 'missing slash' magic
		$urlInstance = $this->testAlternatives($url);

		if ($urlInstance !== null) {
			$this->writeCache($url, $urlInstance);
			return $urlInstance;
		}

		// try all of our configured domains to see whether one matches
		$domains   = Util::getDefaultDomains(true);
		$reqDomain = parse_url($url, PHP_URL_HOST);

		foreach ($domains as $clang => $clangDomains) {
			foreach ($clangDomains as $clangDomain) {
				if ($reqDomain !== $clangDomain) {
					$test        = 'http://'.$clangDomain.mb_substr($url, 7+strlen($reqDomain));
					$urlInstance = $this->findUrl($test);

					if ($urlInstance !== null) {
						if (!$this->cacheHit) {
							$this->writeCache($url, $urlInstance);
						}

						return $urlInstance;
					}

					// try the '/index.html' and 'missing slash' magic as well
					$urlInstance = $this->testAlternatives($test);

					if ($urlInstance !== null) {
						$this->writeCache($url, $urlInstance);
						return $urlInstance;
					}
				}
			}
		}

		// well, we're out of options
		return null;
	}

	protected function testAlternatives($url) {
		// replace '/index.[html|php|...]' (some commonly used suffixes) with '.html'
		$test = preg_replace('#/index\.(html|html|php|phtml|xhtml)$#i', '.html', $url);

		if ($test !== $url) {
			$urlInstance = $this->findUrl($test);

			if ($urlInstance !== null) {
				return $urlInstance;
			}
		}

		// if the URL does not look like a file, maybe someone just forgot the '.html' extension
		if (!preg_match('#\.(html|html|php|phtml|xhtml)$#', $url)) {
			$test        = $url.'.html';
			$urlInstance = $this->findUrl($test);

			if ($urlInstance !== null) {
				return $urlInstance;
			}
		}

		// if someone removed the filename ('/foo/bar.html' => '/foo/') and expects
		// to find the index page, help by appending '.html' ourselves.
		$test = preg_replace('#/$#i', '.html', $url);

		if ($test !== $url) {
			$urlInstance = $this->findUrl($test);

			if ($urlInstance !== null) {
				return $urlInstance;
			}
		}

		// Maybe someone is requesting an old URL where there were still IDs embedded
		// to make them unique. Later, URLs have been re-configured and now there are
		// no IDs anymore. Help those clients by removing the ID automatically
		// (turn '/foo-24.html' into '/foo.html').
		if (preg_match('#^(.*?)(-\d+)(\.html|/)$#', $url, $match)) {
			$test        = $match[1].$match[3];
			$urlInstance = $this->findUrl($test);

			if ($urlInstance !== null) {
				return $urlInstance;
			}
		}

		return null;
	}

	public function findUrl($url, $returnAll = false) {
		// try cache
		$instance = $this->readCache($url);

		if ($instance !== null) {
			$this->cacheHit = true;
			return $instance;
		}

		$this->cacheHit = false;

		// find the URL in our storage
		$urls = $this->storage->findByUrl($url, true);
		if (empty($urls)) return null;

		// if only a single URL was found, cool, move on.
		if (count($urls) === 1) return reset($urls);

		// return all candidates if requested
		if ($returnAll) return $urls;

		// We got a conflict. Damn.
		$foreign = array();
		$own     = array();

		foreach ($urls as $url) {
			if ($url->getNamespace() !== RealURL2::URL_NAMESPACE) {
				$foreign[] = $url;
			}
			else {
				$own[] = $url;
			}
		}

		// Rule 1: non-realURL2 URLs (i.e. addOn URLs) have priority
		if (count($foreign) === 1) {
			return $foreign[0];
		}
		elseif (count($foreign) > 1) {
			usort($foreign, array($this, 'compareURLByPath'));
			return reset($foreign);
		}

		// Rule 2: sort by structure depth; deeper articles have priority
		return $this->resolveConflicts($own);
	}

	public function resolveConflicts(array $urls) {
		$posMap = array();
		$urlMap = array();
		$maxLen = 0;

		foreach ($urls as $url) {
			$article = sly_Util_Article::findById($url->getArticleID(), $url->getClang());
			if (!$article) continue;

			$positions = $this->computePositions($article);
			$ident     = $url->getIdent();

			$posMap[$ident] = $positions;
			$urlMap[$ident] = $url;

			if (count($positions) > $maxLen) {
				$maxLen = count($positions);
			}
		}

		// well, no valid URL, i.e. article, found
		if (count($posMap) === 0) {
			return null;
		}

		for ($level = 0; $level < $maxLen; ++$level) {
			$maxPos = 0;

			// get highest priority in this level
			foreach ($posMap as $posList) {
				if (isset($posList[$level]) && $posList[$level] > $maxPos) {
					$maxPos = $posList[$level];
				}
			}

			// remove urls with lower priotity or those that aren't on the current level anymores
			foreach ($posMap as $ident => $posList) {
				if (!isset($posList[$level]) || $posList[$level] < $maxPos) {
					unset($posMap[$ident]);
				}
			}

			if (count($posMap) === 1) {
				reset($posMap);
				$ident = key($posMap);

				return $urlMap[$ident];
			}
			elseif (count($posMap) === 0) {
				break;
			}
		}

		// check if we have multiple languages
		// It's (very) possible someone forgot to define a distinct base URI for all languages
		// and has therefore created ambiguous homepage URLs.

		$clangs = array();

		foreach ($urls as $url) {
			if (!in_array($url->getClang(), $clangs)) {
				$clangs[] = $url->getClang();
			}
		}

		if (count($clangs) > 1) {
			throw new Exception('Could not resolve the language to use with the given URL. Did you define proper, distinct base URIs for all languages?');
		}

		throw new Exception('Conflict resolution ended without a result URL. This should never happen.');
	}

	public function computePositions(sly_Model_Article $article) {
		$path      = Util::decodePath($article->getPath());
		$clang     = $article->getClang();
		$positions = array();

		foreach ($path as $id) {
			$positions[] = $this->getPosition($id, $clang);
		}

		// add own position
		$positions[] = $this->getPosition($article->getId(), $clang);

		return $positions;
	}

	protected function getPosition($id, $clang) {
		$cat      = sly_Util_Article::findById($id, $clang);
		$parentId = $cat->getParentId();

		// select all elements on the same level
		$pos = 1;

		$this->db->select('article', 'id', array('re_id' => $parentId, 'clang' => $clang), null, 'startpage DESC, catpos ASC, pos ASC');

		foreach ($this->db->all() as $row) {
			if ($row['id'] == $id) {
				return $pos;
			}

			++$pos;
		}

		throw new Exception('Article '.$id.' does not live inside it\'s re_id '.$parentId.'. This should never happen.');
	}

	public function compareURLByPath(URL $a, URL $b) {
		// sort descending
		return strcmp($b->getPath(), $a->getPath());
	}

	protected function readCache($url) {
		return $this->cache->get('realurl2.volatile', md5($url));
	}

	protected function writeCache($url, URL $urlInstance) {
		$this->cache->set('realurl2.volatile', md5($url), $urlInstance);
	}
}
